/**
 * Module description
 * @moduleName DailyRewardManager
 * @snippet DailyRewardManager.snippet
DailyRewardManager({
    position: new Vector2(0, 0)
})
 */
bento.define('modules/dailyrewardmanager', [
    'bento',
    'bento/math/vector2',
    'bento/math/rectangle',
    'bento/components/sprite',
    'bento/components/clickable',
    'bento/entity',
    'bento/eventsystem',
    'bento/gui/clickbutton',
    'bento/gui/counter',
    'bento/gui/text',
    'bento/utils',
    'bento/tween',
    'modules/timemanager',
    'modules/localization'
], function (
    Bento,
    Vector2,
    Rectangle,
    Sprite,
    Clickable,
    Entity,
    EventSystem,
    ClickButton,
    Counter,
    Text,
    Utils,
    Tween,
    TimeManager,
    Localization
) {
    'use strict';
    // --- VARS ---
    var maxStreak = 7;
    var today;
    var lastDayCollected;
    var dayStreak;
    var weekStreak;
    var canCollectReward = false;

    var weeksRewards;
    var todaysReward = {
        id: null,
        name: '',
        value: null,
        canDouble: false
    };

    // --- REWARDS ---
    var rewards;

    // --- FUNCTIONS ---
    var checkDailyReward = function () {
        today = Math.floor(TimeManager.getCurrentLocalTime() / 1000 / 60 / 60 / 24);
        lastDayCollected = Bento.saveState.load('dailyLoginLastDayCollected', 0);
        dayStreak = Bento.saveState.load('dailyLoginDayStreak', 0);
        weekStreak = Bento.saveState.load('dailyLoginWeekStreak', 0);

        // is this a new day
        if (today > lastDayCollected) {
            // we can collect our reward
            canCollectReward = true;
            // if we reached the max last collection, reset
            if (dayStreak >= maxStreak) {
                //reset the day streak
                dayStreak = 0;
                Bento.saveState.save('dailyLoginCurrentStreak', dayStreak);
                // up the weak streak
                weekStreak++;
                Bento.saveState.save('dailyLoginWeekStreak', weekStreak);
            }
        }

        //get todays reward
        weeksRewards = rewards[weekStreak + 1] || rewards['default'];
        todaysReward = weeksRewards[dayStreak + 1] || todaysReward;

        //print stuff
        console.log('Daily Reward Initialized! can we collect a reward: ', canCollectReward);

        //cancel event
        EventSystem.off('serverTimeUpdated', checkDailyReward);
    };

    // --- MODULE ---
    var DailyRewardManager = {
        initialize: function () {
            //set up rewards now
            rewards = {
                "default": {
                    "1": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "2": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "3": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "4": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "5": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "6": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "7": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "100"),
                        value: 100,
                        canDouble: true
                    },
                },
                "1": {
                    "1": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "2": {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "lemontea",
                        canDouble: false
                    },
                    "3": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "4": {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "bubbletea",
                        canDouble: false
                    },
                    "5": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "6": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "7": {
                        id: 'plant',
                        name: Localization.getText('potato-plant'),
                        value: "potato",
                        canDouble: false
                    }
                },
                "2": {
                    "1": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "25"),
                        value: 25,
                        canDouble: true
                    },
                    "2": {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "medicine",
                        canDouble: false
                    },
                    "3": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "35"),
                        value: 35,
                        canDouble: true
                    },
                    "4": {
                        id: 'pot',
                        name: Localization.getText('freePot'),
                        value: "soymilk",
                        canDouble: false
                    },
                    "5": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "6": {
                        id: 'coins',
                        name: Localization.getText('XCoins').replace('{COUNT}', "50"),
                        value: 50,
                        canDouble: true
                    },
                    "7": {
                        id: 'plant',
                        name: Localization.getText('tomato-plant'),
                        value: "tomato",
                        canDouble: false
                    }
                }
            };

            // do we have server time?
            if (TimeManager.hasUpdatedTime) {
                // yes? initialize
                checkDailyReward();
            } else {
                // no? wait for it
                EventSystem.on('serverTimeUpdated', checkDailyReward);
            }
        },
        canCollectReward: function () {
            return canCollectReward;
        },
        getWeeksRewards: function () {
            return weeksRewards;
        },
        getTodaysReward: function () {
            return todaysReward;
        },
        getDayStreak: function () {
            return dayStreak;
        },
        collect: function (handleReward) {
            if (canCollectReward) {
                //we can't collect any more
                canCollectReward = false;

                //save that we collected today
                lastDayCollected = today;
                Bento.saveState.save('dailyLoginLastDayCollected', lastDayCollected);

                //up our streak
                dayStreak++;
                Bento.saveState.save('dailyLoginDayStreak', dayStreak);

                //create dialog for reward
                if (handleReward) {
                    handleReward(todaysReward);
                }

                // print stuff
                console.log('Daily Reward Collected!');
            }
        }

    };
    return DailyRewardManager;
});